@@ -39,22 +39,29 @@ enum {
 	Q_RESIZE = 1 << 5,	// resize the queue if needed
 };
 
+/* MAX_AUDIT_MESSAGE_LENGTH, aligned to 4 KB so that an average q_append() only
+   writes to two disk disk blocks (1 aligned data block, 1 header block). */
+#define QUEUE_ENTRY_SIZE (3*4096)
+
+/* Close Q. */
+void q_close(struct queue *q);
+
 /* Open a queue using Q_FLAGS and return it. If Q_IN_FILE: use PATH for the
  * file, NUM_ENTRIES must be the same for all users of the file unless Q_RESIZE
  * is set. ENTRY_SIZE is the maximum length of a stored string, including the
  * trailing NUL. If Q_IN_FILE, it must be the same for all users of the file.
  * On error, return NULL and set errno. */
 struct queue *q_open(int q_flags, const char *path, size_t num_entries,
-		     size_t entry_size);
-/* Close Q. */
-void q_close(struct queue *q);
+		     size_t entry_size)
+	__attribute_malloc__ __attr_dealloc (q_close, 1) __wur;
 
 /* Add DATA to tail of Q. Return 0 on success, -1 on error and set errno. */
 int q_append(struct queue *q, const char *data);
 
 /* Peek at head of Q, storing it into BUF of SIZE. Return 1 if an entry 
  * exists, 0 if queue is empty. On error, return -1 and set errno. */
-int q_peek(struct queue *q, char *buf, size_t size);
+int q_peek(struct queue *q, char *buf, size_t size)
+	__attr_access ((__write_only__, 2, 3));
 
 /* Drop head of Q and return 0. On error, return -1 and set errno. */
 int q_drop_head(struct queue *q);
