@@ -1,5 +1,5 @@
 /* auparse.h --
- * Copyright 2006-08,2012,2014-17,2022 Red Hat Inc.
+ * Copyright 2006-08,2012-23 Red Hat Inc.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -12,9 +12,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; see the file COPYING. If not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor
+ * Boston, MA 02110-1335, USA.
  *
  * Authors:
  *      Steve Grubb <sgrubb@redhat.com>
@@ -61,12 +62,13 @@ int auparse_feed(auparse_state_t *au, const char *data, size_t data_len)
 	__attr_access ((__read_only__, 2, 3));
 void auparse_feed_age_events(auparse_state_t *au);
 int auparse_flush_feed(auparse_state_t *au);
-int auparse_feed_has_data(auparse_state_t *au);
+int auparse_feed_has_data(const auparse_state_t *au);
 int auparse_feed_has_ready_event(auparse_state_t *au);
 void auparse_add_callback(auparse_state_t *au, auparse_callback_ptr callback,
 			void *user_data, user_destroy user_destroy_func);
 void auparse_set_escape_mode(auparse_state_t *au, auparse_esc_t mode);
 int auparse_reset(auparse_state_t *au);
+char *auparse_metrics(const auparse_state_t *au);
 
 /* Functions that are part of the search interface */
 int ausearch_add_expression(auparse_state_t *au, const char *expression,
@@ -92,7 +94,7 @@ int auparse_set_eoe_timeout (time_t new_tmo);
 int auparse_normalize(auparse_state_t *au, normalize_option_t opt);
 
 // Event kind accessor
-const char *auparse_normalize_get_event_kind(auparse_state_t *au);
+const char *auparse_normalize_get_event_kind(const auparse_state_t *au);
 
 // session accessor
 int auparse_normalize_session(auparse_state_t *au);
@@ -100,12 +102,12 @@ int auparse_normalize_session(auparse_state_t *au);
 // Subject accessing functions
 int auparse_normalize_subject_primary(auparse_state_t *au);
 int auparse_normalize_subject_secondary(auparse_state_t *au);
-const char *auparse_normalize_subject_kind(auparse_state_t *au);
+const char *auparse_normalize_subject_kind(const auparse_state_t *au);
 int auparse_normalize_subject_first_attribute(auparse_state_t *au);
 int auparse_normalize_subject_next_attribute(auparse_state_t *au);
 
 // Action string accessor
-const char *auparse_normalize_get_action(auparse_state_t *au);
+const char *auparse_normalize_get_action(const auparse_state_t *au);
 
 // Object accessing functions
 int auparse_normalize_object_primary(auparse_state_t *au);
@@ -113,13 +115,13 @@ int auparse_normalize_object_secondary(auparse_state_t *au);
 int auparse_normalize_object_primary2(auparse_state_t *au);
 int auparse_normalize_object_first_attribute(auparse_state_t *au);
 int auparse_normalize_object_next_attribute(auparse_state_t *au);
-const char *auparse_normalize_object_kind(auparse_state_t *au);
+const char *auparse_normalize_object_kind(const auparse_state_t *au);
 
 // Results accessor
 int auparse_normalize_get_results(auparse_state_t *au);
 
 // How accessor
-const char *auparse_normalize_how(auparse_state_t *au);
+const char *auparse_normalize_how(const auparse_state_t *au);
 
 // Syscall key accessor
 int auparse_normalize_key(auparse_state_t *au);
@@ -129,49 +131,49 @@ int ausearch_next_event(auparse_state_t *au);
 int auparse_next_event(auparse_state_t *au);
 
 /* Accessors to event data */
-const au_event_t *auparse_get_timestamp(auparse_state_t *au);
-time_t auparse_get_time(auparse_state_t *au);
-unsigned int auparse_get_milli(auparse_state_t *au);
-unsigned long auparse_get_serial(auparse_state_t *au);
-const char *auparse_get_node(auparse_state_t *au) __attr_dealloc_free;
-int auparse_node_compare(au_event_t *e1, au_event_t *e2);
-int auparse_timestamp_compare(au_event_t *e1, au_event_t *e2);
-unsigned int auparse_get_num_records(auparse_state_t *au);
+const au_event_t *auparse_get_timestamp(const auparse_state_t *au);
+time_t auparse_get_time(const auparse_state_t *au);
+unsigned int auparse_get_milli(const auparse_state_t *au);
+unsigned long auparse_get_serial(const auparse_state_t *au);
+const char *auparse_get_node(const auparse_state_t *au) __attr_dealloc_free;
+int auparse_node_compare(const au_event_t *e1, const au_event_t *e2);
+int auparse_timestamp_compare(const au_event_t *e1, const au_event_t *e2);
+unsigned int auparse_get_num_records(const auparse_state_t *au);
 
 /* Functions that traverse records in the same event */
 int auparse_first_record(auparse_state_t *au);
 int auparse_next_record(auparse_state_t *au);
-unsigned int auparse_get_record_num(auparse_state_t *au);
-int auparse_goto_record_num(auparse_state_t *au, unsigned int num);
+unsigned int auparse_get_record_num(const auparse_state_t *au);
+int auparse_goto_record_num(const auparse_state_t *au, unsigned int num);
 
 /* Accessors to record data */
-int auparse_get_type(auparse_state_t *au);
-const char *auparse_get_type_name(auparse_state_t *au);
-unsigned int auparse_get_line_number(auparse_state_t *au);
-const char *auparse_get_filename(auparse_state_t *au);
-int auparse_first_field(auparse_state_t *au);
-int auparse_next_field(auparse_state_t *au);
-unsigned int auparse_get_num_fields(auparse_state_t *au);
-const char *auparse_get_record_text(auparse_state_t *au);
-const char *auparse_get_record_interpretations(auparse_state_t *au);
+int auparse_get_type(const auparse_state_t *au);
+const char *auparse_get_type_name(const auparse_state_t *au);
+unsigned int auparse_get_line_number(const auparse_state_t *au);
+const char *auparse_get_filename(const auparse_state_t *au);
+int auparse_first_field(const auparse_state_t *au);
+int auparse_next_field(const auparse_state_t *au);
+unsigned int auparse_get_num_fields(const auparse_state_t *au);
+const char *auparse_get_record_text(const auparse_state_t *au);
+const char *auparse_get_record_interpretations(const auparse_state_t *au);
 const char *auparse_find_field(auparse_state_t *au, const char *name);
-const char *auparse_find_field_next(auparse_state_t *au);
-unsigned int auparse_get_field_num(auparse_state_t *au);
-int auparse_goto_field_num(auparse_state_t *au, unsigned int num);
+const char *auparse_find_field_next(const auparse_state_t *au);
+unsigned int auparse_get_field_num(const auparse_state_t *au);
+int auparse_goto_field_num(const auparse_state_t *au, unsigned int num);
 
 /* Accessors to field data */
-const char *auparse_get_field_name(auparse_state_t *au);
-const char *auparse_get_field_str(auparse_state_t *au);
-int auparse_get_field_type(auparse_state_t *au);
-int auparse_get_field_int(auparse_state_t *au);
+const char *auparse_get_field_name(const auparse_state_t *au);
+const char *auparse_get_field_str(const auparse_state_t *au);
+int auparse_get_field_type(const auparse_state_t *au);
+int auparse_get_field_int(const auparse_state_t *au);
 const char *auparse_interpret_field(auparse_state_t *au);
-const char *auparse_interpret_realpath(auparse_state_t *au);
+const char *auparse_interpret_realpath(const auparse_state_t *au);
 const char *auparse_interpret_sock_family(auparse_state_t *au);
 const char *auparse_interpret_sock_port(auparse_state_t *au);
 const char *auparse_interpret_sock_address(auparse_state_t *au);
 #ifdef __cplusplus
 }
 #endif
- 
+
 #endif
 
