@@ -89,6 +89,7 @@
 #include "famtabs.h"
 #include "fcntl-cmdtabs.h"
 #include "flagtabs.h"
+#include "fsconfigs.h"
 #include "ipctabs.h"
 #include "ipccmdtabs.h"
 #include "mmaptabs.h"
@@ -324,7 +325,7 @@ static void key_escape(const char *orig, char *dest, auparse_esc_t escape_mode)
 static int is_int_string(const char *str)
 {
 	while (*str) {
-		if (!isdigit(*str))
+		if (!isdigit((unsigned char)*str))
 			return 0;
 		str++;
 	}
@@ -365,6 +366,8 @@ char *au_unescape(char *buf)
 	// strlen(buf) / 2.
 	olen = strlen(buf);
 	str = malloc(olen+1);
+	if (!str)
+		return NULL;
 
         saved = *ptr;
         *ptr = 0;
@@ -1482,7 +1485,7 @@ static const char *print_success(const char *val)
 {
         int res;
 
-	if (isdigit(*val)) {
+	if (isdigit((unsigned char)*val)) {
 	        errno = 0;
 		res = strtoul(val, NULL, 10);
 	        if (errno) {
@@ -1831,6 +1834,28 @@ static const char *print_mount(const char *val)
 	return strdup(buf);
 }
 
+static const char *print_fsconfig(const char *val)
+{
+	char *out;
+	const char *s;
+	int cmd;
+
+	errno = 0;
+	cmd = strtoul(val, NULL, 16);
+	if (errno) {
+		if (asprintf(&out, "conversion error(%s)", val) < 0)
+			out = NULL;
+		return out;
+	}
+
+	s = fsconfig_i2s(cmd);
+	if (s != NULL)
+		return strdup(s);
+	if (asprintf(&out, "unknown-fsconfig-operation(%d)", cmd) < 0)
+		out = NULL;
+	return out;
+}
+
 static const char *print_rlimit(const char *val)
 {
 	int i;
@@ -1929,7 +1954,6 @@ static char *print_dirfd(const char *val)
 	errno = 0;
 	uint32_t i = strtoul(val, NULL, 16);
 	if (errno) {
-		char *out;
 		if (asprintf(&out, "conversion error(%s)", val) < 0)
 			out = NULL;
 		return out;
@@ -2277,7 +2301,7 @@ static const char *print_ioctl_req(const char *val)
 	return out;
 }
 
-static const char *fanotify[3]= { "unknown", "allow", "deny" };
+static const char *fanotify[3] = { "unknown", "allow", "deny" };
 static const char *aulookup_fanotify(unsigned s)
 {
 	switch (s)
@@ -2295,7 +2319,7 @@ static const char *print_fanotify(const char *val)
 {
         int res;
 
-	if (isdigit(*val)) {
+	if (isdigit((unsigned char)*val)) {
 	        errno = 0;
 		res = strtoul(val, NULL, 10);
 	        if (errno) {
@@ -2393,7 +2417,7 @@ static const char *print_trust(const char *val)
 	return out;
 }
 
-// fan_type always preceeds fan_info
+// fan_type always precedes fan_info
 static int last_type = 2;
 static const char *print_fan_type(const char *val)
 {
@@ -2557,6 +2581,15 @@ static const char *print_a1(const char *val, const idata *id)
 				return print_mode_short(val, 16);
 			else if (strncmp(sys, "fcntl", 5) == 0)
 				return print_fcntl_cmd(val);
+			else if (strncmp(sys, "fsconfig", 5) == 0)
+				return print_fsconfig(val);
+			else if (strncmp(sys, "fsopen", 6) == 0) {
+				if (strcmp(val, "1") == 0)
+					return strdup("FSOPEN_CLOEXEC");
+			} else if (strncmp(sys, "fsmount", 7) == 0) {
+				if (strcmp(val, "1") == 0)
+					return strdup("FSMOUNT_CLOEXEC");
+			}
 		} else if (*sys == 'c') {
 			if (strcmp(sys, "chmod") == 0)
 				return print_mode_short(val, 16);
@@ -2675,6 +2708,8 @@ static const char *print_a2(const char *val, const idata *id)
 				return print_mode_short(val, 16);
 			else if (strncmp(sys, "faccessat", 9) == 0)
 				return print_access(val);
+			else if (strncmp(sys, "fsmount", 7) == 0)
+				return print_mount(val);
 		} else if (*sys == 's') {
 			if (strcmp(sys, "setresuid") == 0)
 				return print_uid(val, 16);
@@ -2695,6 +2730,8 @@ static const char *print_a2(const char *val, const idata *id)
 				return print_mode_short(val, 16);
 			else if (strcmp(sys, "mprotect") == 0)
 				return print_prot(val, 0);
+			else if (strcmp(sys, "move_mount") == 0)
+				return print_dirfd(val);
 			else if ((strcmp(sys, "mq_open") == 0) &&
 						(id->a1 & O_CREAT))
 				return print_mode_short(val, 16);
