@@ -15,7 +15,8 @@
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING. If not, write to the
-* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor 
+* Boston, MA 02110-1335, USA.
 *
 * Authors:
 *   Steve Grubb <sgrubb@redhat.com>
@@ -48,8 +49,8 @@ void list_create(llist *l);
 void list_first(llist *l);
 void list_last(llist *l);
 lnode *list_next(llist *l);
-static inline lnode *list_get_cur(llist *l) { return l->cur; }
-void list_append(llist *l, struct audit_rule_data *r, size_t sz);
+static inline lnode *list_get_cur(const llist *l) { return l->cur; }
+void list_append(llist *l, const struct audit_rule_data *r, size_t sz);
 void list_clear(llist* l);
 
 #endif
