@@ -40,6 +40,8 @@
 #include <libgen.h>	/* For basename */
 #include <limits.h>	/* PATH_MAX */
 #include <signal.h>
+#include <sys/syscall.h> // SYS_pidfd_open
+#include <poll.h>
 #include "libaudit.h"
 #include "auditctl-listing.h"
 #include "private.h"
@@ -288,6 +290,8 @@ static int audit_setup_watch_name(struct audit_rule_data **rulep, char *path)
 	if (audit_add_watch_dir(type, rulep, path)) 
 		return -1;
 
+	if (add != AUDIT_FILTER_UNSET)
+		audit_msg(LOG_INFO, "Old style watch rules are slower");
 	return 1;
 }
 
@@ -295,7 +299,7 @@ static int audit_setup_watch_name(struct audit_rule_data **rulep, char *path)
  * Setup a watch permissions.
  * Returns a 1 on success & -1 on failure.
  */
-static int audit_setup_perms(struct audit_rule_data *rule, const char *opt)
+static int audit_setup_perms(const char *opt)
 {
 	unsigned int i, len, val = 0;
 
@@ -402,6 +406,46 @@ static int check_rule_mismatch(int lineno, const char *option)
 	return 0;
 }
 
+#ifdef SYS_pidfd_open
+static int pidfd_open(int pid, unsigned int flags)
+{
+	return syscall(SYS_pidfd_open, pid, flags);
+}
+
+static int pidfd_send_signal(int pidfd, int sig, siginfo_t *info,
+			     unsigned int flags)
+{
+	return syscall(SYS_pidfd_send_signal, pidfd, sig, info, flags);
+}
+
+// This function uses the new pidfd_ family of functions to send
+// the signal to auditd. If the signal is SIGTERM, it waits for auditd
+// to exit before returning. This is to prevent old and new daemons
+// from stepping on each other since auditd shutsdown slowly.
+static int sure_kill(int pid, int signal)
+{
+	int rc = 0;
+	int pidfd = pidfd_open(pid, 0);
+	pidfd_send_signal(pidfd, signal, NULL, 0);
+	if (signal == SIGTERM) {
+		struct pollfd pollfd;
+		pollfd.fd = pidfd;
+		pollfd.events = POLLIN;
+		int ready = poll(&pollfd, 1, -1);
+		if (ready == -1) {
+			perror("poll");
+			rc = -1;
+			goto out;
+		}
+		// Check if it exited or errored
+		if (!(pollfd.revents & POLLIN))
+			rc = -1;
+	}
+out:
+	close(pidfd);
+	return rc;
+}
+#endif
 
 static int send_signal(const char *optarg)
 {
@@ -431,7 +475,7 @@ static int send_signal(const char *optarg)
 
 	if (signal == 0) {
 		audit_msg(LOG_ERR, "%s is an unsupported signal", optarg);
-		return -1;
+		exit(1);
 	}
 
 	// Request status so that we can find the pid
@@ -439,7 +483,7 @@ static int send_signal(const char *optarg)
 	if (retval == -1) {
 		if (errno == ECONNREFUSED)
 			audit_msg(LOG_INFO, "The audit system is disabled");
-		return -1;
+		exit(1);
 	}
 
 	// Receive the netlink info
@@ -469,21 +513,25 @@ static int send_signal(const char *optarg)
 				if (rep.status->pid == 0) {
 					audit_msg(LOG_INFO,
 						"Auditd is not running");
-					return -2;
+					exit(2);
 				}
+#ifdef SYS_pidfd_open
+				retval = sure_kill(rep.status->pid, signal);
+#else
 				retval = kill(rep.status->pid, signal);
+#endif
 				if (retval < 0) {
 					audit_msg(LOG_WARNING,
 				        "Failed sending signal to auditd (%s)",
 						 strerror(errno));
-					return -1;
+					exit(1);
 				} else
 					return -2;
 			}
 		}
 	}
 	audit_msg(LOG_WARNING, "Failed sending signal to auditd (timeout)");
-	return -1;
+	exit(1);
 }
 
 static int report_status(void)
@@ -508,45 +556,15 @@ static int report_status(void)
 	return -2;
 }
 
-static int parse_syscall(const char *optarg)
-{
-	int retval = 0;
-	char *saved;
-
-	if (strchr(optarg, ',')) {
-		char *ptr, *tmp = strdup(optarg);
-		if (tmp == NULL)
-			return -1;
-		ptr = strtok_r(tmp, ",", &saved);
-		while (ptr) {
-			retval = audit_rule_syscallbyname_data(rule_new, ptr);
-			if (retval != 0) {
-				if (retval == -1) {
-					audit_msg(LOG_ERR,
-						"Syscall name unknown: %s", 
-						ptr);
-					retval = -3; // error reported
-				}
-				break;
-			}
-			ptr = strtok_r(NULL, ",", &saved);
-		}
-		free(tmp);
-		return retval;
-	}
-
-	return audit_rule_syscallbyname_data(rule_new, optarg);
-}
-
 #ifdef WITH_IO_URING
-// return 0 on success and -1 if unknow op.
+// return 0 on success and -1 if unknown op.
 static int parse_io_uring(const char *optarg)
 {
 	if (strchr(optarg, ',')) {
-		int retval;
+		int retval = -1;
 		char *saved, *ptr, *tmp = strdup(optarg);
 		if (tmp == NULL)
-			return -1;
+			return retval;
 		ptr = strtok_r(tmp, ",", &saved);
 		while (ptr) {
 			retval = audit_rule_io_uringbyname_data(rule_new, ptr);
@@ -561,7 +579,7 @@ static int parse_io_uring(const char *optarg)
 }
 #endif
 
-static struct option long_opts[] =
+static const struct option long_opts[] =
 {
 #if HAVE_DECL_AUDIT_FEATURE_VERSION == 1
   {"loginuid-immutable", 0, NULL, 1},
@@ -662,7 +680,7 @@ static int setopt(int count, int lineno, char *vars[])
 		}
 		break;
         case 'r':
-		if (optarg && isdigit(optarg[0])) { 
+		if (optarg && isdigit((unsigned char)optarg[0])) {
 			uint32_t rate;
 			errno = 0;
 			rate = strtoul(optarg,NULL,0);
@@ -681,7 +699,7 @@ static int setopt(int count, int lineno, char *vars[])
 		}
 		break;
         case 'b':
-		if (optarg && isdigit(optarg[0])) {
+		if (optarg && isdigit((unsigned char)optarg[0])) {
 			uint32_t limit;
 			errno = 0;
 			limit = strtoul(optarg,NULL,0);
@@ -872,7 +890,7 @@ static int setopt(int count, int lineno, char *vars[])
 				_audit_elf = elf;
 			}
 		}
-		rc = parse_syscall(optarg);
+		rc = _audit_parse_syscall(optarg, rule_new);
 		switch (rc)
 		{
 			case 0:
@@ -1071,7 +1089,7 @@ process_keys:
 			audit_msg(LOG_ERR, "permission option needs a filter");
 			retval = -1;
 		} else 
-			retval = audit_setup_perms(rule_new, optarg);
+			retval = audit_setup_perms(optarg);
 		break;
         case 'q':
 		if (_audit_syscalladded) {
@@ -1116,7 +1134,7 @@ process_keys:
 	case 2:
 #if HAVE_DECL_AUDIT_VERSION_BACKLOG_WAIT_TIME == 1 || \
     HAVE_DECL_AUDIT_STATUS_BACKLOG_WAIT_TIME == 1
-		if (optarg && isdigit(optarg[0])) {
+		if (optarg && isdigit((unsigned char)optarg[0])) {
 			uint32_t bwt;
 			errno = 0;
 			bwt = strtoul(optarg,NULL,0);
@@ -1263,7 +1281,7 @@ static void preprocess(char *buf)
 }
 
 
-void postprocess(char *buf)
+static void postprocess(char *buf)
 {
 	char *str = strdup(buf);
 	char *pos1 = str;
